/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*******************************************************************************
* MPIA, NEVEC - MIDI project
*
* "@(#) $Id: parser.c,v 1.4 2002/05/08 21:34:35 jdejong Exp $"
*
* who        when      what
* ---------  --------  ----------------------------------------------
* jdejong    2002-05-08 imported
*
*/
/* $Id: parser.c,v 1.4 2002/05/08 21:34:35 jdejong Exp $ */
/* source file for input parser */
#include "parser.h"

/* internal prototypes */
/* delete a parse_item */
void kill_parse_item (parse_item* item); /* item to destroy */
/* parse line into keyword and value part */
void parse_line(char *line,char * keyword, char *value);



/************** external interface ***************************/

parse_list* create_parse_list() 
/*--------------------------------------------------------------------*/
/*  Create parser list                                                */
/*   Returns:  Empty parser_list                                      */
/*--------------------------------------------------------------------*/
{
  parse_list *out;

  out = (parse_list*)malloc(sizeof(parse_list));
  if (out==NULL)     {  /* check for allocation failure */
    fprintf(stderr,"ERROR creating parse_list\n");
    return out;
  }
  memset (out, 0, sizeof(parse_list)); /* zero fill */
  
  /* fill in values */
  out->NumItem = 0;
  return out;
} /* end create_parse_list */

void kill_parse_list(parse_list* list) 
/*--------------------------------------------------------------------*/
/*  Delete parser list                                                */
/*   Inputs:                                                          */
/*     list  Parse_list structure to delete                           */
/*--------------------------------------------------------------------*/
{
  long i;
  if (!list) return; /* bad pointer? */

  /* delete all elements */
  for (i=0; i<list->NumItem; i++) {
    kill_parse_item(list->list[i]);
  }
  /* debug 
     return;*/
  free (list);
} /* end kill_parse_list */

int parse_input_file(char *name,
		     parse_list *list)
/*--------------------------------------------------------------------*/
/*  Parse values from input file into list.                           */
/*   Inputs:                                                          */
/*      name    name of input file                                    */
/*      list    parse list structure                                  */
/*   Returns:    0 on success, else failure                           */
/*--------------------------------------------------------------------*/
{
  FILE *fp;
  char line[201], keyword[50], value[100];
  long i;
  int found, bad=0, iret;

  /* open file */
  fp = fopen(name,"r");
  if (fp == NULL) {
    fprintf(stderr,"Cannot open input file %s\n",name);
    return 1;
  }

  /* loop over file */
  while (fgets (line, 200, fp)) {
    printf("%s",line); /* debug */
    if (line[0]=='#') continue; /* ignore comment lines */
    
    /* parse line into keyword and value part */
    parse_line(line, keyword, value);

    /* find in parser list */
    found = 0;
    for (i=0; i<list->NumItem; i++)
      {
	if (!strcmp(keyword, list->list[i]->name)) { /* found a match? */
	  /* read value into list element */
	  switch (list->list[i]->type) {
	  case (PINTEGER):
	    iret = sscanf (value,"%ld",&list->list[i]->ivalue);
	    break;
	  case (PFLOAT):
	    iret = sscanf (value,"%lf",&list->list[i]->fvalue);
	    break;
	  case (PSTRING):
	    if (list->list[i]->cvalue) free(list->list[i]->cvalue);
	    list->list[i]->cvalue = malloc(strlen(value)+1);
	    strcpy(list->list[i]->cvalue, value);
	    iret = strlen(value);
	    break;
	  default:
	    iret = 0;
	    break;
	  }; /* end switch */
	  /* make sure value read */
	  if (iret<1) {
	    fprintf (stderr,"ERROR parsing input line: %s\n",line); 
	    return 1;
	  }
	  found = 1;
	  break;
	} /* end found match */
      }
    /* complain if not found */
    if (!found) {
      fprintf (stderr, "ERROR invalid input parameter %s\n", keyword);
      bad = 1;
    }

  } /* end loop over file */
  if (bad) {
      fprintf (stderr, "Terminating due to bad input\n");
    return 1; /* problem with input */
  }
  return 0;
} /* end  parse_input_file */



parse_item* make_parse_item (char *name,
			     enum parser_type type,
			     Float fvalue, 
			     Integer ivalue,
			     char *cvalue)
/*--------------------------------------------------------------------*/
/*  Create a parse_item from values                                   */
/*   Inputs:                                                          */
/*     name     name of item                                          */
/*     type     item type                                             */
/*     fvalue   Float default value                                   */
/*     ivalue   Integer default value                                 */
/*     cvalue   String default value                                  */
/*   Returns:  parse_item pointer to new structure with values filled */
/*--------------------------------------------------------------------*/
{
  parse_item *out;

  out = (parse_item*)malloc(sizeof(parse_item));
  if (out==NULL)     {  /* check for allocation failure */
    fprintf(stderr,"ERROR creating parse_item\n");
    return out;
  }
  memset (out, 0, sizeof(parse_item)); /* zero fill */
  
  /* fill in values */
  out->name = (char*)malloc (strlen (name)+1);
  strcpy (out->name, name);
  out->type = type;
  out->fvalue = fvalue;
  out->ivalue = ivalue;
  if (cvalue==NULL) {
    out->cvalue = cvalue; /* no value given */
  } else { /* save value */
    out->cvalue = malloc(strlen(cvalue)+1);
    strcpy(out->cvalue, cvalue);
  }
  return out;
} /* end make_parse_item */


int find_parse_entry(char *name, parse_list* list) 
/*--------------------------------------------------------------------*/
/* Find a given entry in a parse list, return index if OK, else -1    */
/*   Inputs:                                                          */
/*      name    name of parse_item                                    */
/*      list    parse list structure                                  */
/*   Returns:   index if OK, else -1                                  */
/*--------------------------------------------------------------------*/
{
  int i;
  for (i=0;  i<list->NumItem; i++)
    {
      if (!strcmp(name, list->list[i]->name)) { /* found a match? */
	return i;
      }
    } /* end of loop over list */

  fprintf (stderr,"WARNING: cannot find parse entry %s\n", name);
  return -1; /* no match */
} /* end find_parse_entry */

/**************** internal routines **************************/


/* delete a parse_item */
void kill_parse_item (parse_item* item)  /* item to destroy */
{
  if (!item) return; /* sanity check */
  if (item->cvalue!=NULL) free (item->cvalue); /* delete string */
  if (item->name) free (item->name);
  free (item);
} /* end  kill_parse_item */

/* parse line into keyword and value part */
void parse_line(char *line, char *keyword, char *value) {
  int i, key_count=0, val_count=0, found=0; /* found says if '=' past */
  
  for (i=0; i<(int)strlen(line); i++) {
    if (line[i]=='#') break; /* comment follows? */
    if (line[i]=='=') { /* '=' divides two parts */
      found = 1;
      continue;
    }
    /* part of keyword? */
    if ((!found) && (line[i]!=' ') && (line[i]!='\n') && 
	(line[i]!='\t')) { /* part of keyword */
      *keyword++ = line[i];
    }
    /* part of value? */
    if (found && (line[i]!=' ') && (line[i]!='\n') && 
	(line[i]!='\t')) { /* part of value */
      *value++ = line[i];
    }
  } /* end loop over line */
  
  /* terminate strings */
  *keyword = 0;
  *value = 0;
} /* end parse_line */

/* */
